/***
 * Semaphore test #2
 *
 * Change Log:
 * 03/10/22 ghort	created
 * 03/11/03 pallas	cleaned up and integrated
 * 03/11/04 pallas	made the semaphore array dynamically allocated
 * 03/11/07 pallas	converted to robust_thread_create
 * 04/10/25 pallas	updated for new assignment specification
 * 05/01/13 pallas	updated for user space API
 */

static char *	desc =
	"Semaphore test #2\n"
	"Creates and array of semaphores and sequentially decrements\n"
	"(and sleeps on) them in one thread and increments them in "
	"another.\n\n";


#include "prereq/assignment-3.h"
#include "defs.h"


/*
 * The number of semaphores to create.
 */
#define SEM_COUNT		(TASK_SIZE * 100)


/*
 * Number of running threads and associated mutex.
 */
static uint			threads_running;
static struct mutex		threads_running_mtx;

/*
 * Semaphores...
 */
static struct semaphore *	sem_array [SEM_COUNT];


static void
thread_proc_down (void * data)
{
	int sem;
	assert (data == THREAD_MAGIC);

	for (sem = 0; sem < SEM_COUNT; sem++) {
		sem_down (sem_array [sem]);
	}

	dec_var_mutex (threads_running, threads_running_mtx);
}


static void
thread_proc_up (void * data)
{
	int sem;
	assert (data == THREAD_MAGIC);

	for (sem = 0; sem < SEM_COUNT; sem++) {
		sem_up (sem_array [sem]);

		/*
		 * Yield so that we don't increment more than one
		 * semaphore at a time.
		 */
		thread_yield ();
	}

	dec_var_mutex (threads_running, threads_running_mtx);
}


int
main (void)
{
	int 			cnt;
	int 			success;
	thread_t		up_thread;
	thread_t		dn_thread;
	struct semaphore *	sem_mem;

	printf (desc);

	// init
	success = 1;
	threads_running = 2;

	mutex_init (& threads_running_mtx);

	sem_mem = (struct semaphore *) malloc (
		SEM_COUNT * sizeof (struct semaphore));
	if (! sem_mem) {
		printf ("Unable to allocate memory for semaphore array.\n"
			"Test failed...\n");
		return 0;
	}

	for (cnt = 0; cnt < SEM_COUNT; cnt++) {
		sem_init (sem_mem, 0);
		sem_array [cnt] = sem_mem++;
	}

	sem_mem = sem_array [0];


	/*
	 * Start the threads.
	 */
	dn_thread = robust_thread_create (thread_proc_down, THREAD_MAGIC);
	up_thread = robust_thread_create (thread_proc_up, THREAD_MAGIC);

	/*
	 * Wait for all the threads to finish.
	 */
	cond_wait_mutex (threads_running > 0, threads_running_mtx);

	/*
	 * Reap the threads.
	 */
	robust_thread_join (dn_thread);
	robust_thread_join (up_thread);

	/*
	 * Verify that all the semaphores are at 0.
	 */
	for (cnt = 0; cnt < SEM_COUNT; cnt++) {
		if (sem_get_value (sem_array [cnt]) != 0) {
			success = 0;
		}
	}


	// clean up
	mutex_destroy (& threads_running_mtx);

	for (cnt = 0; cnt < SEM_COUNT; cnt++) {
		sem_destroy (sem_array [cnt]);
	}

	free (sem_mem);

	// print the result
	if (success) {
		printf ("Test passed...\n");
	} else {
		printf ("At least one of the semaphores has nonzero value.\n"
			"Test failed...\n");
	}

	return 0;
}
