/***
 * Thread test #2
 *
 * Change Log:
 * 03/10/22 ghort	created
 * 03/11/03 ghort	cleaned up and integrated
 * 03/11/03 pallas	more cleanups
 * 03/11/07 pallas	converted to robust_thread_create
 * 04/10/25 pallas	updated for new assignment specification
 * 05/01/13 pallas	updated for user space API
 */

static char *	desc =
	"Thread test #2\n"
	"Basic test of thread facilities such as suspend, \n"
	"wakeup, join, detach, and kill.\n\n";


#include "prereq/assignment-3.h"
#include "defs.h"


/*
 * The delay between printing characters in thread.
 */
#define PRINT_DELAY_MS	500

/*
 * The delay between individual steps.
 */
#define STEP_DELAY_MS	1500


void
thread_proc (void *data)
{
	thread_t thread;

	thread = thread_self ();
	printf ("Thread %x started...\n", thread);

	printf ("Thread %x suspending...\n", thread);
	thread_suspend ();

	printf ("Thread %x woke up...\n", thread);

	do {
		printf (".");
		thread_usleep (PRINT_DELAY_MS * 1000);
	} while (1);
}


int
main (void)
{
	int		result;
	thread_t	thread;

	printf (desc);


	/*
	 * Start the thread.
	 */
	printf ("Starting thread...\n");
	thread = robust_thread_create (thread_proc, NULL);

	thread_usleep (STEP_DELAY_MS * 1000);

	/*
	 * Wake up the thread.
	 */
	printf ("Waking up thread...\n");
	robust_thread_wakeup (thread);

	thread_usleep (STEP_DELAY_MS * 1000);

	/*
	 * Detach the thread.
	 */
	printf ("Detaching thread...\n");
	robust_thread_detach (thread);

	thread_usleep (STEP_DELAY_MS * 1000);

	/*
	 * Try to join the thread.
	 */
	printf ("Trying to join the thread...\n");
	result = thread_join (thread);
	if (result != EINVAL) {
		printf ("Should not be able to join detached thread.\n"
			"Test failed\n");
		return 0;
	}

	thread_usleep (STEP_DELAY_MS * 1000);

	/*
	 * Try to detach the thread again.
	 */
	printf ("Trying to detach the thread...\n");
	result = thread_detach (thread);
	if (result != EINVAL) {
		printf ("Should not be able to detach already detached thread.\n"
			"Test failed\n");
		return 0;
	}

	thread_usleep (STEP_DELAY_MS * 1000);

	/*
	 * Kill the thread.
	 */
	printf ("Cancelling thread...\n");
	robust_thread_cancel (thread);

	thread_usleep (STEP_DELAY_MS * 1000);


	// print the result
	printf ("Test passed...\n");
	return 0;
}
